library gen_client;

import 'dart:io';
import 'dart:convert';
import 'dart:typed_data';

import './gen_socket.dart';
import './data_utils.dart';
import './op_codes.dart' as OpCodes;

class GenClient {

  late GenSocket gen_socket;
  late Function(GenSocket gen_socket, String data) onMessageHandler = (GenSocket gen_socket, String data) {};
  late Function(GenSocket gen_socket) onConnectHandler = (GenSocket gen_socket) {};
  late Function(GenSocket gen_socket) onCloseHandler = (GenSocket gen_socket) {};

  String _host = '127.0.0.1';
  int _port = 8080;

  GenClient({String host = '0.0.0.0', int port = 8080}) {
    this._host = host;
    this._port = port;
  }

  void log(String msg) {
    print('[GenClient] $msg');
  }

  void onMessage(Function(GenSocket gen_socket, String data) onMessageHandler) {
    this.onMessageHandler = onMessageHandler;
  }

  void onConnect(Function(GenSocket gen_socket) onConnectHandler) {
    this.onConnectHandler = onConnectHandler;
  }

  void onClose(Function(GenSocket gen_socket) onCloseHandler) {
    this.onCloseHandler = onCloseHandler;
  }

  Future<void> start() async {
    this.gen_socket = GenSocket(host: _host, port: _port);
    await gen_socket.initialize();
    log('Connected to: ${gen_socket.remoteAddress}');
    gen_socket.onMessage(this.onMessageHandler);
    gen_socket.onAfterHandshake(() async {
      await gen_socket.sendHandshakeComplete();
    });
    gen_socket.onConnect(() async {
      log("Connect!!");
      onConnectHandler(gen_socket);
    });
    gen_socket.onClose(() {
      log('Close');
      onCloseHandler(gen_socket);
    });
    gen_socket.start();
    await gen_socket.sendHandshakeInfo();
  }

}
